@@ -1,7 +1,7 @@
 //
 // System object for the Printer Application Framework
 //
-// Copyright © 2019-2024 by Michael R Sweet.
+// Copyright © 2019-2025 by Michael R Sweet.
 // Copyright © 2010-2019 by Apple Inc.
 //
 // Licensed under Apache License v2.0.  See the file "LICENSE" for more
@@ -108,7 +108,7 @@ _papplSystemConfigChanged(
 // - `PAPPL_SOPTIONS_MULTI_QUEUE`: Support multiple printers.
 // - `PAPPL_SOPTIONS_WEB_NETWORK`: Include the network settings web page.
 // - `PAPPL_SOPTIONS_RAW_SOCKET`: Accept jobs via raw sockets starting on port
-//   9100.
+//   9100 (all but Windows).
 // - `PAPPL_SOPTIONS_WEB_REMOTE`: Allow remote queue management.
 // - `PAPPL_SOPTIONS_WEB_SECURITY`: Include the security settings web page.
 // - `PAPPL_SOPTIONS_WEB_INTERFACE`: Include the standard printer and job monitoring
@@ -175,6 +175,10 @@ papplSystemCreate(
   pthread_mutex_init(&system->subscription_mutex, NULL);
   pthread_cond_init(&system->subscription_cond, NULL);
 
+#if _WIN32
+  options &= (pappl_soptions_t)~PAPPL_SOPTIONS_RAW_SOCKET;
+#endif // _WIN32
+
   system->options           = options;
   system->start_time        = time(NULL);
   system->name              = strdup(name);
@@ -236,7 +240,9 @@ papplSystemCreate(
 
   system->log_is_syslog = !strcmp(system->log_file, "syslog");
 
-  _papplLogOpen(system);
+  pthread_mutex_lock(&system->log_mutex);
+  _papplLogOpenNoLock(system);
+  pthread_mutex_unlock(&system->log_mutex);
 
   // Initialize authentication...
   if (system->auth_service && !strcmp(system->auth_service, "none"))
@@ -517,6 +523,8 @@ papplSystemRun(pappl_system_t *system)	// I - System
     return;
   }
 
+  papplLog(system, PAPPL_LOGLEVEL_DEBUG, "Server header value is '%s'.", system->server_header);
+
   // Make the static attributes...
   make_attributes(system);
 
@@ -542,11 +550,27 @@ papplSystemRun(pappl_system_t *system)	// I - System
     {
       pthread_t	tid;			// Thread ID
 
+      papplLogPrinter(printer, PAPPL_LOGLEVEL_DEBUG, "Starting socket listener thread.");
+
       if (pthread_create(&tid, &tattr, (void *(*)(void *))_papplPrinterRunRaw, printer))
       {
 	// Unable to create listener thread...
 	papplLogPrinter(printer, PAPPL_LOGLEVEL_ERROR, "Unable to create raw listener thread: %s", strerror(errno));
       }
+      else
+      {
+	// Detach the main thread from the raw thread to prevent hangs...
+	pthread_detach(tid);
+
+        _papplRWLockRead(printer);
+	while (!printer->raw_active)
+	{
+	  _papplRWUnlock(printer);
+	  usleep(1000);			// Wait for raw thread to start
+	  _papplRWLockRead(printer);
+	}
+	_papplRWUnlock(printer);
+      }
     }
   }
 
@@ -555,20 +579,40 @@ papplSystemRun(pappl_system_t *system)	// I - System
   {
     pthread_t	tid;			// Thread ID
 
+    papplLogPrinter(printer, PAPPL_LOGLEVEL_DEBUG, "Starting USB listener thread.");
+
     if (pthread_create(&tid, &tattr, (void *(*)(void *))_papplPrinterRunUSB, printer))
     {
       // Unable to create USB thread...
       papplLogPrinter(printer, PAPPL_LOGLEVEL_ERROR, "Unable to create USB gadget thread: %s", strerror(errno));
     }
+    else
+    {
+      // Detach the main thread from the USB thread to prevent hangs...
+      pthread_detach(tid);
+
+      _papplRWLockRead(printer);
+      while (!printer->usb_active)
+      {
+	_papplRWUnlock(printer);
+	usleep(1000);			// Wait for USB thread to start
+	_papplRWLockRead(printer);
+      }
+      _papplRWUnlock(printer);
+    }
   }
 
   // Loop until we are shutdown or have a hard error...
+  papplLog(system, PAPPL_LOGLEVEL_DEBUG, "Entering run loop.");
+
   for (;;)
   {
     if (restart_logging)
     {
       restart_logging = false;
-      _papplLogOpen(system);
+      pthread_mutex_lock(&system->log_mutex);
+      _papplLogOpenNoLock(system);
+      pthread_mutex_unlock(&system->log_mutex);
     }
 
     gettimeofday(&curtime, NULL);
@@ -680,6 +724,8 @@ papplSystemRun(pappl_system_t *system)	// I - System
     {
       system->save_changes = system->config_changes;
 
+      papplLog(system, PAPPL_LOGLEVEL_DEBUG, "Saving state/config changes.");
+
       if (system->save_cb)
       {
         // Save the configuration...
@@ -691,9 +737,6 @@ papplSystemRun(pappl_system_t *system)	// I - System
     if (system->shutdown_time || sigterm_time)
     {
       // Shutdown requested, see if we can do so safely...
-      cups_len_t	jcount = 0;	// Number of active jobs
-
-      // Force shutdown after 60 seconds
       if (system->shutdown_time && (time(NULL) - system->shutdown_time) > 60)
       {
         _papplRWUnlock(system);
@@ -711,13 +754,12 @@ papplSystemRun(pappl_system_t *system)	// I - System
       {
 	printer = (pappl_printer_t *)cupsArrayGetElement(system->printers, i);
 
-        _papplRWLockRead(printer);
-        jcount += cupsArrayGetCount(printer->active_jobs);
-        _papplRWUnlock(printer);
+        if (printer->processing_job)
+          break;
       }
       _papplRWUnlock(system);
 
-      if (jcount == 0)
+      if (i >= count)
         break;
     }
     else
@@ -788,6 +830,8 @@ papplSystemRun(pappl_system_t *system)	// I - System
   {
     system->save_changes = system->config_changes;
 
+    papplLog(system, PAPPL_LOGLEVEL_DEBUG, "Saving state/config changes.");
+
     if (system->save_cb)
     {
       // Save the configuration...
@@ -798,6 +842,8 @@ papplSystemRun(pappl_system_t *system)	// I - System
   if ((system->options & PAPPL_SOPTIONS_USB_PRINTER) && (printer = papplSystemFindPrinter(system, NULL, system->default_printer_id, NULL)) != NULL)
   {
     // Wait for the USB gadget thread(s) to complete...
+    papplLogPrinter(printer, PAPPL_LOGLEVEL_DEBUG, "Waiting for USB listener thread to exit.");
+
     _papplRWLockRead(printer);
     while (printer->usb_active)
     {
@@ -807,6 +853,8 @@ papplSystemRun(pappl_system_t *system)	// I - System
     }
     _papplRWUnlock(printer);
   }
+
+  papplLog(system, PAPPL_LOGLEVEL_DEBUG, "System run loop completed.");
 }
 
 
